/**
 *  fsNXF
 *
 *  fs(Complex ̂Qz)ɑ΂e퉉Z\bh񋟂܂B
 *
 *  @version $Revision: 1.9 $, $Date: 2003/10/30 19:53:06 $
 */
public class CMatrix {
    /**
     *  s i Ԗڂ̍s j Ԗڂ̍s
     *  @param  i, j    sԍ(0,1,..)
     */
    public static void swapRow(Complex[][] mtx, int i, int j) {
        Complex[] temp = mtx[i]; mtx[i] = mtx[j]; mtx[j] = temp;
    }
    /**
     *  s i Ԗڂ̗ j Ԗڂ̗
     *  @param  i, j    ԍ(0,1,..)
     */
    public static void swapCol(Complex[][] mtx, int i, int j) {
        for (int k = 0; k < mtx.length; k++) {
            Complex temp = mtx[k][i]; mtx[k][i] = mtx[k][j]; mtx[k][j] = temp;
        }
    }
    /**
     *  ]usԂ܂B
     *  @return mtx<sup>T</sup>
     */
    public static Complex[][] transpose(Complex[][] mtx) {
        int n = mtx.length, m = mtx[0].length;
        Complex[][] ret = new Complex[m][n];
        for (int i = 0; i < n; i++) for (int j = 0; j < m; j++)
            ret[j][i] = new Complex(mtx[i][j]);
        return ret;
    }
    /**
     *  fsԂ܂B
     *  @return mtx<sup>*</sup>
     */
    public static Complex[][] conj(Complex[][] mtx) {
        int n = mtx.length, m = mtx[0].length;
        Complex[][] ret = new Complex[n][m];
        for (int i = 0; i < n; i++) for (int j = 0; j < m; j++)
            ret[i][j] = mtx[i][j].conj();
        return ret;
    }
    /**
     *  G~[gsԂ܂B
     *  @return mtx<sup>+</sup>
     */
    public static Complex[][] hermiteConj(Complex[][] mtx) {
        int n = mtx.length, m = mtx[0].length;
        Complex[][] ret = new Complex[m][n];
        for (int i = 0; i < n; i++) for (int j = 0; j < m; j++)
            ret[j][i] = mtx[i][j].conj();
        return ret;
    }
    /**
     *  l (lhs + rhs) ƂȂsԂ܂B
     *  @param  lhs 
     *  @param  rhs E
     *  @return (lhs + rhs)
     */
    public static Complex[][] add(Complex[][] lhs, Complex[][] rhs) {
        int n = lhs.length, m = lhs[0].length;
        if (n != rhs.length || m != rhs[0].length)
            throw new IndexOutOfBoundsException();
        Complex[][] ret = new Complex[n][m];
        for (int i = 0; i < n; i++) for (int j = 0; j < m; j++)
            ret[i][j] = lhs[i][j].add(rhs[i][j]);
        return ret;
    }
    /**
     *  l (lhs - rhs) ƂȂsԂ܂B
     *  @param  lhs 
     *  @param  rhs E
     *  @return (lhs - rhs)
     */
    public static Complex[][] subtract(Complex[][] lhs, Complex[][] rhs) {
        int n = lhs.length, m = lhs[0].length;
        if (n != rhs.length || m != rhs[0].length)
            throw new IndexOutOfBoundsException();
        Complex[][] ret = new Complex[n][m];
        for (int i = 0; i < n; i++) for (int j = 0; j < m; j++)
            ret[i][j] = lhs[i][j].subtract(rhs[i][j]);
        return ret;
    }
    /**
     *  l (lhs * rhs) ƂȂsԂ܂B
     *  @param  lhs 
     *  @param  rhs E
     *  @return (lhs * rhs)
     */
    public static Complex[][] multiply(Complex[][] lhs, Complex[][] rhs) {
        int n = lhs.length, m = rhs[0].length, l = rhs.length;
        if (l != lhs[0].length) throw new IndexOutOfBoundsException();
        Complex[][] ret = new Complex[n][m];
        for (int i = 0; i < n; i++) for (int j = 0; j < m; j++) {
            double re = 0.0, im = 0.0;
            for (int k = 0; k < l; k++) {
                Complex value = lhs[i][k].multiply(rhs[k][j]);
                re += value.re(); im += value.im();
            }
            ret[i][j] = new Complex(re,im);
        }
        return ret;
    }
    /**
     *  l (mtx * v) ƂȂxNgԂ܂B
     *  @param  mtx s
     *  @param  v   xNg
     *  @return (mtx * v)
     */
    public static Complex[] multiply(Complex[][] mtx, Complex[] v) {
        int n = mtx.length, m = mtx[0].length;
        if (m != v.length) throw new IndexOutOfBoundsException();
        Complex[] ret = new Complex[n];
        for (int i = 0; i < n; i++) {
            double re = 0.0, im = 0.0;
            for(int j = 0; j < m; j++) {
                Complex value = mtx[i][j].multiply(v[j]);
                re += value.re(); im += value.im();
            }
            ret[i] = new Complex(re,im);
        }
        return ret;
    }
    /**
     *  l (v * mtx) ƂȂxNgԂ܂B
     *  @param  v   xNg
     *  @param  mtx s
     *  @return (v * mtx)
     */
    public static Complex[] multiply(Complex[] v, Complex[][] mtx) {
        int n = mtx.length, m = mtx[0].length;
        if (n != v.length) throw new IndexOutOfBoundsException();
        Complex[] ret = new Complex[m];
        for (int i = 0; i < m; i++) {
            double re = 0.0, im = 0.0;
            for(int j = 0; j < n; j++) {
                Complex value = v[j].multiply(mtx[j][i]);
                re += value.re(); im += value.im();
            }
            ret[i] = new Complex(re,im);
        }
        return ret;
    }
    /**
     *  s̓e𕶎ŏo͂܂B
     *  @param  mtx s
     *  @return "[[mtx[0][0], mtx[0][1], ...], [mtx[1][0], ...], ...]"
     *          `̕
     */
    public static String toString(Complex[][] mtx) {
        int n = mtx.length, m = mtx[0].length;
        String str = "[";
        for (int i = 0; i < n; i++) {
            str += "[";
            for (int j = 0; j < m; j++) {
                str += mtx[i][j];
                if (j < m - 1) str += ", ";
            }
            str += "]";
            if (i < n - 1) str += ", ";
        }
        str += "]";
        return str;
    }
    /**
     *  B̃Gg|Cg
     *  @param  args    --
     */
    public static void main(String[] args) {
        Complex[][]  m1 = new Complex[2][2];
        m1[0][0] = new Complex(1.0,1.0); m1[0][1] = new Complex(1.0,1.0);
        m1[1][0] = new Complex(1.0,1.0); m1[1][1] = new Complex(1.0,1.0);
        Complex[][]  m2 = (Complex[][])m1.clone(); // vf̓Rs[ȂI
        Complex[][]  m3 = add(m1,m2), m4 = multiply(m2,m3),
                     m5 = subtract(m3,m4);
        System.out.println(toString(m3) + "\n"
                         + toString(m4) + "\n"
                         + toString(m5));
    }
}

