/*
* Jacobi.java -- Jacobi (Rr) @
* @version $Revision: 1.2 $, $Date: 2003/03/13 14:24:32 $
* class RMatrix, RVector gB
*/
import java.io.*;
/**
*Jacobi (Rr) @̃vOłB
*@see Tridiagonalize
*@see RVector
*@see RMatrix
*/

public class Jacobi {
    static final double EPS      = 1E-6;  /* e덷 */
    static final double TINY     = 1E-20; /* 0 ƌȂĂ悢l */
    static final int    MAX_ITER = 100;   /* ő̌JԂ */

    static void rotate(double[][] a, int i, int j, int k, int l, double c, double s) {
        double x = a[i][j], y = a[k][l];
        a[i][j] = x * c - y * s;
        a[k][l] = x * s + y * c;
    }
    /**
    *<code>a[0..n-1][0..n-1]</code> ɓΏ̍s A ̌ŗLl
    *ŗLxNgJacobi@ŋ߂܂B<br>
    *<code>a[k][k]</code>ɌŗLlCɑΉŗLxNg 
    *<code>w[k][0..n-1]</code> ɏ㏑܂B
    *@return  trueC false
    */
    public static boolean solve(double[][] a, double[][] w) {
        int n = a.length;
        double s = 0, offdiag = 0;

        for (int j = 0; j < n; j++) {
            for (int k = 0; k < n; k++) w[j][k] = 0;
            w[j][j] = 1;  s += a[j][j] * a[j][j];
            for (int k = j + 1; k < n; k++)
                offdiag += a[j][k] * a[j][k];
        }
        double tolerance = EPS * EPS * (s / 2 + offdiag);
        int iter;

        for (iter = 1; iter <= MAX_ITER; iter++) {
            offdiag = 0;
            for (int j = 0; j < n - 1; j++)
                for (int k = j + 1; k < n; k++)
                    offdiag += a[j][k] * a[j][k];
// eXgp //////̗lq\////////
                System.out.println( iter + " : " +
                    Math.sqrt(2 * offdiag / (n * (n - 1))));
//////////////////
            if (offdiag < tolerance) break;
            for (int j = 0; j < n - 1; j++) {
                for (int k = j + 1; k < n; k++) {
                    if (Math.abs(a[j][k]) < TINY) continue;
                    double t = (a[k][k] - a[j][j]) / (2 * a[j][k]);
                    if (t >= 0) t = 1 / (t + Math.sqrt(t * t + 1));
                    else        t = 1 / (t - Math.sqrt(t * t + 1));
                    double c = 1 / Math.sqrt(t * t + 1), u = t * c;
                    t *= a[j][k];
                    a[j][j] -= t;  a[k][k] += t;  a[j][k] = 0;
                    for (int i = 0; i < j; i++)     rotate(a, i, j, i, k, c, u);
                    for (int i = j + 1; i < k; i++) rotate(a, j, i, i, k, c, u);
                    for (int i = k + 1; i < n; i++) rotate(a, j, i, k, i, c, u);
                    for (int i = 0; i < n; i++)     rotate(w, j, i, k, i, c, u);
                }
            }
        }
        if (iter > MAX_ITER) return false;  /*  */
        for (int i = 0; i < n - 1; i++) {
            int k = i;  double t = a[k][k];
            for (int j = i + 1; j < n; j++)
                if (a[j][j] > t) {  k = j;  t = a[k][k];  }
            a[k][k] = a[i][i];  a[i][i] = t;
            double[] v = w[k];  w[k] = w[i];  w[i] = v;
        }
        return true;  /*  */
    }

    /**
    * gvOłB<br>
    *<b>&quot;n = &quot;</b>ƕĂ܂ 2 ȏ̐l()͂ĂB<br>
    *̍s AC̗lqCŗLlƌŗLxNgCŗLxNg̒𐫁C敽ό덷
    *o͂܂B
    */

    public static void main(String[] args) throws IOException {
        BufferedReader in = new
            BufferedReader(new InputStreamReader(System.in));

        System.out.print("n = ");
        int n = Integer.parseInt(in.readLine());
        double[][] a = new double[n][n], b = new double[n][n],
                   w = new double[n][n];

        for (int i = 0; i < n; i++)
            for (int j = 0; j <= i; j++)
                a[i][j] = a[j][i] = b[i][j] = b[j][i] = Math.random() - Math.random();
        System.out.println("̍s A = ");
        RMatrix.print(a, 7, "10.6f");
        System.out.println("̗lq");
        if (! solve(a, w)) System.out.println("܂");

        double[] lambda = new double[n];
        for (int i = 0; i < n; i++) lambda[i] = a[i][i];
//        System.out.println("ŗLl:");    RVector.print(lambda, 7, "10.6f");
        System.out.println();
        for (int i = 0; i < n; i++) {
            System.out.println("ŗLl lambda[" + i +"] = " + lambda[i]);
            System.out.println("ŗLxNg w[" + i + "] =");
            RVector.print(w[i], 7, "10.6f");
        }
        System.out.println("\nŗLxNg̒𐫂̊mF");
        for (int i = 0; i < n; i++)
            for (int j = i + 1; j < n; j++)
                System.out.println("w[" + i + "]Ew[" + j + "] = "
                    + RVector.innerProduct(w[i], w[j]));

        double err = 0;
        for (int i = 0; i < n; i++){
            for (int j = 0; j < n; j++){
                double s = b[i][j];
                for (int k = 0; k < n; k++) s -= lambda[k] * w[k][i] * w[k][j];
                err += s * s;
            }
        }
        System.out.println("\n敽ό덷: " + Math.sqrt(err / (n * n)));
    }
}
