/**
 *  Maze.java -- H
 *  (DrawOneByOnePanel.java g)
 *
 *  @version $Revision: 1.15 $, $Date: 2003/03/18 10:05:54 $
 */

import java.awt.*;
import java.util.Random;
import java.util.ArrayList;

public class Maze extends DrawOneByOnePanel {
    static final int X_MAX = 80;  // H̉̑傫 ()
    static final int Y_MAX = 40;  // H̏c̑傫 ()
    static final int H = 10; // ʘHƕǂ̕
    static final int WIDTH = (X_MAX - 3) * H, HEIGHT = (Y_MAX - 3) * H;

    static final int WALL    = 0;  // 
    static final int PASSAGE = 1;  // ʘH
    static final int VISITED = 2;  // ̒ʘH
    static final int ANSWER  = 3;  // 

    static final Color color[] = {
        Color.BLACK,            // ǂ̐F (Java v1.4ł Color.black)
        Color.LIGHT_GRAY,       // ʘH̐F ( Color.lightGray)
        Color.GREEN.darker(),   // ̒ʘH̐F ( Color.green.darker())
        Color.YELLOW,           // ̐F ( Color.yellow)
    };

    static final int DX[] = { 2, 0, -2,  0 };  // ψʃxNg
    static final int DY[] = { 0, 2,  0, -2 };  // ψʃxNg
    static final int DIR_TABLE[][] = {         // \
        {0,1,2,3}, {0,1,3,2}, {0,2,1,3}, {0,2,3,1}, {0,3,1,2}, {0,3,2,1},
        {1,0,2,3}, {1,0,3,2}, {1,2,0,3}, {1,2,3,0}, {1,3,0,2}, {1,3,2,0},
        {2,0,1,3}, {2,0,3,1}, {2,1,0,3}, {2,1,3,0}, {2,3,0,1}, {2,3,1,0},
        {3,0,1,2}, {3,0,2,1}, {3,1,0,2}, {3,1,2,0}, {3,2,0,1}, {3,2,1,0},
    };

    private int map[][] = new int[X_MAX + 1][Y_MAX + 1]; // n}
    private ArrayList site = new ArrayList();
    private Random random = new Random();
    private boolean made = false;
    private Graphics2D g;

    public Maze() {  super(WIDTH, HEIGHT, 0);  }

    private void addSite(int x, int y) {  // TCgɉ
        site.add(new Point(x, y));
    }

    private Point selectSite() {  // TCg𗐐őI
        if (site.isEmpty()) return null;  // TCgs
        int index = random.nextInt(site.size());
        return (Point)(site.remove(index));
    }

    private int nextDir(int x, int y) {  // 󂫒nT
        int[] t = DIR_TABLE[random.nextInt(DIR_TABLE.length)];
        for (int d = 3; d >= 0; d--) {
            int dx = DX[t[d]],  dy = DY[t[d]];
            if (map[x + dx][y + dy] == PASSAGE) return t[d];
        }
        return -1;
    }

    private void setMap(int x, int y, int m) {  // n}ɋL^
        map[x][y] = m;  g.setColor(color[m]);
        g.fillRect((x - 2) * H, (y - 2) * H, H, H);
    }

    synchronized public void drawSlowly(Graphics2D g) {
        this.g = g;
        if (!made) {  make();   made = true;  }
        else       {  solve();  made = false;  }
    }

    private void make() {
        for (int x = 0; x <= X_MAX; x++)   // n}
            for (int y = 0; y <= Y_MAX; y++) setMap(x, y, WALL);
        for (int x = 3; x <= X_MAX - 3; x++)
            for (int y = 3; y <= Y_MAX - 3; y++) setMap(x, y, PASSAGE);
        setMap(2, 3, PASSAGE);  // Əo
        setMap(X_MAX - 2, Y_MAX - 3,  PASSAGE);
        for (int x = 4; x <= X_MAX - 4; x += 2) {  // TCg
            addSite(x, 2);  addSite(x, Y_MAX - 2);
        }
        for (int y = 4; y <= Y_MAX - 4; y += 2) {
            addSite(2, y);  addSite(X_MAX - 2, y);
        }
        sleep();

        Point p;  int d;
        while ((p = selectSite()) != null)  // TCgI
            while ((d = nextDir(p.x, p.y)) >= 0) { // 牄΂Ă
                setMap(p.x + DX[d] / 2, p.y + DY[d] / 2, WALL);
                p.x += DX[d];  p.y += DY[d];  addSite(p.x, p.y);
                setMap(p.x, p.y, WALL);  sleep(30);  // \ăEGCg
            }
    }

    /*
      H
      QlF͐YwJavaɂASYxiZp]_ЁC2001Nj
    */

    boolean scanned[][] = new boolean[X_MAX + 1][Y_MAX + 1];
    boolean success;

    private void solve() {
        success = false;
        for (int x = 0; x <= X_MAX ; x++)
            for (int y = 0; y <= Y_MAX; y++) {
                scanned[x][y] = false;  setMap(x, y, map[x][y]);
            }
        visit(X_MAX - 2, Y_MAX - 3);  // oT
        repaint();
    }

    private void visit(int i, int j) {  // ċAIɒT
        if (success || scanned[i][j] || map[i][j] != PASSAGE) return;
        setMap(i, j, VISITED);  sleep(10);  // ̒ʘH\
        scanned[i][j] = true;
        if (i == 2 && j == 3) success = true;  // ɓ
        visit(i, j - 1); // 
        visit(i, j + 1); // 
        visit(i + 1, j); // E
        visit(i - 1, j); // 
        if (success) {  setMap(i, j, ANSWER);  sleep(30);  }
    }

    public static void main(String[] args) {
        Maze maze = new Maze();
        maze.makeFrame("H");
    }
}
