/**
 *  Newton1.java -- Newton (j[g) @
 *
 *  @version $Revision: 1.12 $, $Date: 2003/04/05 15:10:00 $
 */

import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

/**
 * <code><i>f</i>(<i>x</i>) = 0</code>̉
 * Newton (j[g) @gċ߂܂B
 * <br>
 * @see Cardano
 * @see Newton1Test
 * @see Newton2
 */
public class Newton1 {
    /** _߂֐ f(x) */
    public interface Function {
        public double of(double x);       // ֐ f(x)
        public double primeOf(double x);  // ֐ f'(x)
    }

    static final double EPS = 1.0e-14;
    static final DecimalFormat DF2  = new DecimalFormat(" 0.0E00;-0.0E00");
    static final DecimalFormat DF15 = new
        DecimalFormat(" 0.000000000000000E00;-0.000000000000000E00");

    /** l $x$ ^ $f(x) = 0$ ̉Ԃ */
    public static double solve(Function f, double x) {
        double xPrev;
        do {
            double fx = f.of(x), fp;
            System.out.println("  x = " + DF15.format(x) +
                               "  f(x) = " + DF2.format(fx));
            if ((fp = f.primeOf(x)) == 0) fp = 1;  // ɂ炷
            xPrev = x;  x -= fx / fp;
        } while (Math.abs(x - xPrev) > EPS);
        return x;
    }

    /**
     * W a, b, c, d 󔒂ŋ؂ē͂ƂR<br>
     * <code>ax<sup>3</sup> + bx<sup>2</sup> + cx + d = 0</code><br>
     * ̎\܂B
     * @see Cardano
     */
    public static void main(String[] args) throws IOException {
        /** f(x) = x^3 + bx^2 + c^x + d */
        class CubicFunction implements Newton1.Function {
            private double b, c, d;
            public CubicFunction(double a, double b, double c, double d) {
                this(b / a, c / a, d / a);
            }
            public CubicFunction(double b, double c, double d) {
                this.b = b;  this.c = c;  this.d = d;
            }
            public double of(double x) { // _߂֐ $f(x)$
                return ((b + x) * x + c) * x + d;
            }
            public double primeOf(double x) { // $f(x)$ ̓֐
                return (2 * b + 3 * x) * x + c;
            }
        }

        System.out.println("ax^3+bx^2+cx+d=0܂.");
        System.out.println("a b c d ? ");
        BufferedReader in =
            new BufferedReader(new InputStreamReader(System.in));
        StringTokenizer st = new StringTokenizer(in.readLine());
        double a = Double.parseDouble(st.nextToken()),
               b = Double.parseDouble(st.nextToken()),
               c = Double.parseDouble(st.nextToken()),
               d = Double.parseDouble(st.nextToken());
        if (a == 0) {
            System.out.println("3ł͂܂B");  System.exit(-1);
        }
        b /= a;  c /= a;  d /= a;
        a = b * b - 3 * c;
        CubicFunction cf = new CubicFunction(b, c, d);

        if (a > 0) {
            double mp = (-b + Math.sqrt(a))/3.0;    // ɏ_̍W
            a = (2.0 / 3.0) * Math.sqrt(a);
            double x1 = Newton1.solve(cf, -a - b / 3);  // Cɏ_̒Jn邱Ƃł
            System.out.println("x1 = " + x1);
            if (cf.of(mp) > 0) System.exit(0);      // ɏl :̔肪Ȃ 2 5 2 6 ȂǂŎȂ
            double x2 = Newton1.solve(cf, a - b / 3);   // E
            if (x2 == x1) System.exit(0);
            System.out.println("x2 = " + x2);
            double x3 = Newton1.solve(cf, b / (-3));    // ϋȓ_
            System.out.println("x3 = " + x3);
        } else {
            double x1 = Newton1.solve(cf, 0);           // Kȓ_
            System.out.println("x = " + x1);
        }
    }
}
